﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using Tuitio.Domain.Entities;
using System.Collections.Generic;
using dto = Tuitio.PublishedLanguage.Dto;
using models = Tuitio.Domain.Models;

namespace Tuitio.Application.Mappings
{
    public class MappingProfile : Profile
    {
        public MappingProfile()
        {
            CreateMap<models.Token, dto.AuthorizationResult>();
            CreateMap<AppUser, models.Token>()
                .ForMember(z => z.Claims, src => src.MapFrom(z => ComposeClaims(z.Claims)));

            CreateMap<models.Account.LogoutResult, dto.AccountLogoutResult>();
        }

        private Dictionary<string, string> ComposeClaims(ICollection<UserClaim> claims)
        {
            if (claims == null)
                return null;

            var result = new Dictionary<string, string>();
            foreach (var claim in claims)
                result.Add(claim.ClaimKey, claim.ClaimValue);

            return result;
        }
    }
}
