﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using Newtonsoft.Json;
using System;
using System.Text;
using System.Text.RegularExpressions;
using Tuitio.Application.Services.Abstractions;
using Tuitio.Domain.Abstractions;
using Tuitio.Domain.Entities;
using Tuitio.Domain.Models;

namespace Tuitio.Application.Services
{
    internal class TokenService : ITokenService
    {
        private readonly IMapper _mapper;
        private readonly IConfigProvider _configProvider;

        public TokenService(IMapper mapper, IConfigProvider configProvider)
        {
            _mapper=mapper;
            _configProvider=configProvider;
        }

        public Token GenerateToken(AppUser user)
        {
            var currentDate = DateTime.UtcNow;
            var validUntil = currentDate.AddMinutes(_configProvider.Token.ValidityInMinutes);

            var token = _mapper.Map<Token>(user);
            token.TokenId = Guid.NewGuid();
            token.LockStamp = Regex.Replace(Convert.ToBase64String(Guid.NewGuid().ToByteArray()), "[/+=]", "");
            token.CreatedAt = currentDate;
            token.ExpiresIn = (validUntil - currentDate).TotalMilliseconds;
            return token;
        }

        public string GenerateTokenRaw(Token token)
        {
            var tokenString = JsonConvert.SerializeObject(token);
            var tokenBytes = Encoding.UTF8.GetBytes(tokenString);
            var tokenRaw = Convert.ToBase64String(tokenBytes);
            return tokenRaw;
        }

        public Token ExtractToken(string tokenRaw)
        {
            var valid = ValidateTokenRaw(tokenRaw);
            if (!valid)
                return null;

            var tokenBytes = Convert.FromBase64String(tokenRaw);
            var tokenString = Encoding.UTF8.GetString(tokenBytes);
            var token = JsonConvert.DeserializeObject<Token>(tokenString);
            return token;
        }

        private bool ValidateTokenRaw(string tokenRaw)
        {
            if (string.IsNullOrWhiteSpace(tokenRaw))
                return false;

            if (!StringIsBase64(tokenRaw))
                return false;

            return true;
        }

        private bool StringIsBase64(string str)
        {
            str = str.Trim();
            return (str.Length % 4 == 0) && Regex.IsMatch(str, @"^[a-zA-Z0-9+/]*={0,3}$", RegexOptions.None);
        }
    }
}
