﻿// Copyright (c) 2020 Tudor Stanciu

using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Tuitio.Domain.Data.DbContexts;
using Tuitio.Domain.Data.Repositories;
using Tuitio.Domain.Repositories;

namespace Tuitio.Domain.Data
{
    public static class DependencyInjectionExtensions
    {
        public static void AddDataAccessServices(this IServiceCollection services)
        {
            services.AddScoped<IUserRepository, UserRepository>();
        }

        public static void AddDataAccess(this IServiceCollection services)
        {
            services.AddDataAccessServices();
            services
                .AddDbContextPool<TuitioDbContext>(
                 (serviceProvider, options) =>
                 {
                     var configuration = serviceProvider.GetService<IConfiguration>();
                     var connectionString = configuration.GetConnectionString("DatabaseConnection");
                     options.UseSqlServer(connectionString);
                 });
        }
    }
}
