﻿// Copyright (c) 2020 Tudor Stanciu

using MediatR;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;
using Tuitio.Application.CommandHandlers;

namespace Tuitio.Controllers
{
    [ApiController]
    [Route("account")]
    public class AccountController : ControllerBase
    {
        private readonly IMediator _mediator;

        public AccountController(IMediator mediator)
        {
            _mediator = mediator;
        }

        [HttpPost("login")]
        public async Task<IActionResult> Login([FromQuery] string userName, string password)
        {
            var command = new AccountLoginHandler.Command(userName, password);
            var result = await _mediator.Send(command);
            return Ok(result);
        }

        [HttpPost("logout")]
        public async Task<IActionResult> Logout([FromQuery] string token)
        {
            var command = new AccountLogoutHandler.Command(token);
            var result = await _mediator.Send(command);
            return Ok(result);
        }
    }
}
